#!/env/bin/python3

"""

    The MIT License (MIT)

    Copyright (c) 2014 MARTIN Damien

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
    THE SOFTWARE.

"""

from hashlib import md5
import argparse
import os

# ======================================================================================================================

class Dyff:

    def __init__(self, folder_a, folder_b, output, recursive=True):

        self.a = folder_a
        self.b = folder_b
        self.current_folder = ""
        self.output = output
        self.recursive = recursive

        log = open(self.output, "w+")
        log.write("")

    # ------------------------------------------------------------------------------------------------------------------

    def write_in_log(self, message):

        log = open(self.output, "a")
        log.write("{}\n".format(message))

    # ------------------------------------------------------------------------------------------------------------------

    def scan_folder(self, folder):

        self.current_folder = folder

        current_folder_a = os.path.join(self.a, self.current_folder)
        current_folder_b = os.path.join(self.b, self.current_folder)

        if not os.path.exists(current_folder_b):

            self.write_in_log("- {}".format(current_folder_b))

        else:

            for entry in os.listdir(current_folder_a):

                if os.path.isdir(os.path.join(current_folder_a, entry)):

                    self.scan_folder(os.path.join(folder, entry))

                else:

                    if not os.path.exists(os.path.join(current_folder_b, entry)):

                        self.write_in_log("- {}".format(os.path.join(current_folder_b, entry)))

                    else:

                        hash_entry_a = md5()
                        hash_entry_a.update(open(os.path.join(current_folder_a, entry)).read().encode('utf-8'))

                        hash_entry_b = md5()
                        hash_entry_b.update(open(os.path.join(current_folder_b, entry)).read().encode('utf-8'))

                        if hash_entry_a.digest() != hash_entry_b.digest():

                            self.write_in_log("m {}".format(os.path.join(current_folder_b, entry)))

    # ------------------------------------------------------------------------------------------------------------------

    def run(self):

        print(">>> Starting Dyff")

        self.scan_folder("")

        print(">>> Dyff ended, you can see the result in {}.".format(self.output))

# ======================================================================================================================

def cli():

    parser = argparse.ArgumentParser(description='Look for differences between the files of two folders')
    parser.add_argument("-o", "--output", dest="output", help="The log file to save results.")
    parser.add_argument("folder_a", help="The 'A' folder")
    parser.add_argument("folder_b", help="The 'B' folder")
    args = parser.parse_args()

    if args.output is None:
        output = "dyff.log"
    else:
        output = args.output

    dyff = Dyff(args.folder_a, args.folder_b, output)
    dyff.run()


if __name__ == "__main__":

    cli()